/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2010 The Boeing Company
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Gary Pei <guangyu.pei@boeing.com>
 */

#ifndef DSS_ERROR_RATE_MODEL_H
#define DSS_ERROR_RATE_MODEL_H
#include <stdint.h>

#ifdef ENABLE_GSL
#include <gsl/gsl_math.h>
#include <gsl/gsl_integration.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_sf_bessel.h>
#endif

namespace ns3 {

#ifdef ENABLE_GSL
typedef struct FunctionParameterType
{
 double beta;
 double n;
} FunctionParameters;

double IntegralFunction (double x, void *params);
#endif

/**
 * \brief an implementation of DSSS error rate model
 *
 * The description of these models can be found in
 * http://www.nsnam.org/~pei/80211b.pdf
 */
class DsssErrorRateModel
{
public:
  static double DqpskFunction (double x) ;
  static double GetDsssDbpskSuccessRate (double sinr, uint32_t nbits) ;
  static double GetDsssDqpskSuccessRate (double sinr,uint32_t nbits) ;
  static double GetDsssDqpskCck5_5SuccessRate (double sinr,uint32_t nbits) ;
  static double GetDsssDqpskCck11SuccessRate (double sinr,uint32_t nbits) ;
#ifdef ENABLE_GSL
  static double SymbolErrorProb16Cck (double e2) ; /// equation (18) in Pursley's paper
  static double SymbolErrorProb256Cck (double e1) ; /// equation (17) in Pursley's paper
#else
protected:
  static const double WLAN_SIR_PERFECT;
  static const double WLAN_SIR_IMPOSSIBLE;
#endif
};

} // namespace ns3

#endif /* DSSS_ERROR_RATE_MODEL_H */
